#include <system.h>
#include "50.h"

/*FINAL CONTROL ROUTINE, "PowerCop" Cruiser Power Regulation System
This software performs the following tasks:
	*Checks current being drawn off of auxilliary electrical devices at regular intervals using
	A/D conversion and the known I/V characteristics of our sensors.
	*Checks alternator current, battery current, and battery voltage to estimate whether the batttery is being
	charged or discharged, as well as providing a rough estimate of the state of charge.
	*Shuts off loads depending on the results of these measurements to prevent battery from being overdrawn
	*Writes the current being drawn from the loads, and any potential warnings about battery status and the
	time of each measurement, to an external EEPROM memory chip.
	*Provides a user interface to a series of buttons and an LCD screen, which allows users to set the system
	time, establish thresholds for the various device lines, and check the current being drawn at the moment by a
	given line.
	*Allows the user to connect the system to any PC with a serial port and HyperTerminal and upload the results
	stored in memory.  These results can be easily moved into an Excel spreadsheet.*/

#pragma DATA _CONFIG1H, _OSC_HS_1H //10 mhz
#pragma DATA _CONFIG2H, _WDTPS_16384_2H 
#pragma DATA _CONFIG4L, _LVP_OFF_4L 
#pragma DATA _CONFIG3H, _MCLRE_ON_3H

volatile bit user_int@INTCON3.0;
bool user_flag = false;

volatile bit time_out@INTCON.2;
bool lcd_menu_flag = false;

short timeoutflag = 0; 
short twentyfour = 0;
unsigned short over = 0;
unsigned short term_read = 0;

void interrupt(void)
{
	if (user_int)
	{
		user_int = 0;
		user_flag = true;
	}
	
    
    if (time_out)
    {
		time_out = 0;
		timeoutflag = 1;
		//LCD_setpos(1,0);
		//LCD_printf("Timer0 interrupt");
    
    }         
	
}

void main(void)
{
	unsigned short minutes = 0x09;
	unsigned short hours = 0x13;
	unsigned short date = 0x03;
	unsigned short month = 0x05;

	int_wdt_init();
	LCD_init();
	term_init();
	ad_init();
	i2c_init();
	io_init();
	time_init_clk(minutes, hours, date, month);
	time_init_set();
	
	//time_out_init();
	menu_init();
	/*disable timer o for time being*/
	intcon.5 = 0;
	
	unsigned short feedback = 0;
	bool car_on = true;		//This will trigger an "off" state in control when the unit begins
	unsigned short over = 0;		//System defaults as override
	bool control = false;
	unsigned short fault = 0;
	unsigned short soc = 90;
	//unsigned short term_read = 0;
	bool read_all = false;
	
	unsigned short alt_current = 0;
	
	unsigned short mode;
	unsigned short switch_status;
	unsigned short wait_time = 1;
	unsigned short wait_time_1 = 30;
	unsigned short wait_time_2 = 15;
	unsigned short wait_time_3 = 60;	//Amount of time, minimum, between
	unsigned short time_track = 0;		//Allows system to determine mode of operation on its own shortly after startup
	unsigned short inc = 0;
	unsigned short dummy = 0;
	
	unsigned short size = 22;
	
	unsigned short address_low_start = 0x00;
	unsigned short address_high_start = 0x00;
	unsigned short address_low_inc = 0x00;
	unsigned short address_high_inc = 0x00;
	
	//AD_pin_assignments.  May need to be moved...
	unsigned short L1_ad = 0x01;	//Camera--approx. 7.5 A (Only for monitoring purposes--1)
	unsigned short L2_ad = 0x05;	//GPS Transciever--15 A (best guess--could not tear up car enough to know for sure--3)
	unsigned short L3_ad = 0x09;	//Motorola 2-way radio--15 A (2)
	unsigned short R1_ad = 0x0D;	//Interceptor Main Power / siren (4)
	unsigned short R2_ad = 0x11;	//floods on lightbar, computer, printer (4)
	unsigned short R3_ad = 0x15;	//Power ports (4)
	unsigned short alt_ad = 0x19;	//alternator current
	unsigned short bata_ad = 0x1D;	//battery current
	unsigned short batv_ad = 0x21;	//battery voltage
	
	while(1){
		
		/*If current on alternator is greater than some threshold (car on), set wait_time to
		wait_time_1.  If not (car is off), set it to wait_time_2.*/
		
			LCD_setpos(0,0);
			if(fault == 0 && over == false){
				LCD_printf("System Armed");
			}
			if(fault == 0 && over == true){
				LCD_printf("System Idle");
			}
			if(fault == 1){
				LCD_printf("! Battery SOC @ ");
				LCD_int(soc);
				LCD_printf("% !");
			}
			if(fault == 2){
				LCD_printf("! Alternator Fault !");
			}
			if(fault == 3){
				LCD_printf("ATTN: BATT. FAIL");
				}
			LCD_setpos(1,0);
			dummy = i2c_time(false, 0x00, 0x00);
			int_wdt_init_sleep();
			sleep();
			int_wdt_init();
			if(user_flag == false){
				time_track = time_track + 1;
				if(time_track >= wait_time){
					if (car_on == false){
						wait_time = wait_time_3;
						}
					if (car_on == true){
						wait_time = wait_time_1;
						}
					control = true;
					}
				alt_current = check_ad(alt_ad);
				if((alt_current > 10 && car_on == false) || (alt_current < 10 && car_on == true)){
					if(alt_current > 10){
						car_on = true;
						wait_time = wait_time_2;
						}
					if(alt_current < 10){
						car_on = false;
						soc = soc_check();
						wait_time = wait_time_2;
						}
					control = true;
				}
			}
			if(user_flag == true){
				user_flag = false;
				control = true;
				int_wdt_init();
				LCD_clear();
				LCD_main_menu(read_all, address_high_start, address_low_start, address_high_inc, address_low_inc);	//User interface functions will go here.
				if(term_read == 1){
					address_low_start = address_low_inc;
					address_high_start = address_high_inc;
					term_read = 0;
					read_all = false;
					}
				}
			if(control == true){
				fault = control_routine(over, car_on, wait_time);
				address_low_inc = i2c_time(true, address_low_inc, address_high_inc);
				address_low_inc = i2c_data(car_on, over, fault, address_low_inc, address_high_inc);
				if ((address_low_inc + size) > 255){
					address_low_inc = 0x00;
					address_high_inc = address_high_inc + 1;
					if (address_high_inc == 256){
						address_high_inc = 0;
					}
				}
				if((address_high_inc == address_high_start) && (address_low_inc == address_low_start)){
					read_all = true;
				}
				control = false;
				time_track = 0;
				}
			LCD_clear();
		}
	}



short check_timeoutflag(void){
	return timeoutflag;
	}
	
short write_timeoutflag(short tof){
	timeoutflag = tof;
	}

short check_twentyfour(void){
	return twentyfour;
	}

void write_twentyfour(short tf){
	twentyfour = tf;
	}
	
unsigned short check_override(void){
	return over;
	}

void write_override(unsigned short ovr){
	over = ovr;
}

unsigned short check_termread(){
	return term_read;
}

void write_term_read(unsigned short tr){
	term_read = tr;
}
